using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;
using ZylSoft.Serial;

namespace CSharpDemo
{	
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public partial class Form1 : System.Windows.Forms.Form
	{
												
		private ZylSoft.Serial.SerialPort serialPort;
		
		                     
		private System.ComponentModel.IContainer components;

        // This delegate enables asynchronous calls for setting
        // the text property on a TextBox control.
        delegate void SetTextCallback(string text);

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}		

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

        private void Set_textBoxOut(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.textBoxOut.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(Set_textBoxOut);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.textBoxOut.AppendText(text + Environment.NewLine);
                this.textBoxOut.SelectionStart = textBoxOut.Text.Length;
                this.textBoxOut.SelectionLength = 0;
                this.textBoxOut.ScrollToCaret();
            }
        }


		private void btnOpen_Click(object sender, System.EventArgs e)
		{            
		    serialPort.AutoReconnect = true;
            if (lstPorts.SelectedIndex < 0)
            {
                MessageBox.Show("Select a serial port!");
                return;
            }		    
            serialPort.Port = SerialPort.StringToSerialCommPort(lstPorts.Items[lstPorts.SelectedIndex].ToString());
            if (serialPort.Port == SerialCommPort.Custom)
            {
                serialPort.CustomPortName = lstPorts.Items[lstPorts.SelectedIndex].ToString();
            }            

            serialPort.BaudRate = (SerialBaudRate)(Convert.ToInt32(lstBaudRates.Items[lstBaudRates.SelectedIndex].ToString()));
            serialPort.DataWidth = (SerialDataWidth)(lstDataWidth.SelectedIndex + 5);
            serialPort.ParityBits = (SerialParityBits)(lstParity.SelectedIndex);
            serialPort.StopBits = (SerialStopBits)(lstStopBits.SelectedIndex);
            serialPort.HardwareFlowControl = (SerialHardwareFlowControl)(lstHrdwFlow.SelectedIndex);
            serialPort.SoftwareFlowControl = (SerialSoftwareFlowControl)(lstSoftFlow.SelectedIndex);
            if (checkBoxDTR.Checked)
                serialPort.EnableDtrOnOpen = true;
            else
                serialPort.EnableDtrOnOpen = false;
            if (checkBoxRTS.Checked)
                serialPort.EnableRtsOnOpen = true;
            else
                serialPort.EnableRtsOnOpen = false;

            if (!serialPort.Open())
                MessageBox.Show("Cannot connect to " + serialPort.Port.ToString());
		}

		private void btnClose_Click(object sender, System.EventArgs e)
		{
			serialPort.Close();
		}

		private void btnSend_Click(object sender, System.EventArgs e)
		{
			serialPort.SendAsciiString(txtText.Text + Environment.NewLine);
		}

		private void Form1_Closed(object sender, System.EventArgs e)
		{
			if (serialPort != null)
				serialPort.Close();
		}

		private void serialPort_Sent(object sender, ZylSoft.Serial.DataEventArgs e)
		{
			Set_textBoxOut(textBoxOut.Text + Environment.NewLine  + "Sent data: " + SerialPort.AsciiByteArrayToString(e.Buffer) + Environment.NewLine);			
		}

		private void serialPort_Received(object sender, ZylSoft.Serial.DataEventArgs e)
		{
		    string strBuffer = serialPort.ReadLine();
            byte[] buffer = SerialPort.StringToAsciiByteArray(strBuffer);
            if (rbAscii.Checked)
            {
                if (!checkBoxShowHiddenChars.Checked)
                    Set_textBoxOut(SerialPort.AsciiByteArrayToString(buffer));
                else
                    Set_textBoxOut(SerialPort.ReplaceControlCharacters(SerialPort.AsciiByteArrayToString(buffer)));
            }
            else if (rbDecimal.Checked)
            {
                Set_textBoxOut(SerialPort.ByteArrayToDecimalString(buffer));
            }
            else
            {
                Set_textBoxOut(SerialPort.ByteArrayToHexaString(buffer));
            }				
		}

        private void Form1_Load(object sender, EventArgs e)
        {
            lstBaudRates.SelectedIndex = 9;
            lstDataWidth.SelectedIndex = 3;
            lstParity.SelectedIndex = 0;
            lstStopBits.SelectedIndex = 0;
            lstHrdwFlow.SelectedIndex = 0;
            lstSoftFlow.SelectedIndex = 0;

            lstPorts.Items.Clear();
            String[] ports = ZylSoft.Serial.SerialPort.GetExistingCommPortNames();
            foreach (String i in ports)
            {
                lstPorts.Items.Add(i);  
            }
        }

        private void btnApply_Click(object sender, EventArgs e)
        {
            if (serialPort.ConnectedTo != SerialCommPort.None)
            {
                serialPort.BaudRate = (SerialBaudRate)(Convert.ToInt32(lstBaudRates.Items[lstBaudRates.SelectedIndex].ToString()));
                serialPort.DataWidth = (SerialDataWidth)(lstDataWidth.SelectedIndex + 5);
                serialPort.ParityBits = (SerialParityBits)(lstParity.SelectedIndex);
                serialPort.StopBits = (SerialStopBits)(lstStopBits.SelectedIndex);
                serialPort.HardwareFlowControl = (SerialHardwareFlowControl)(lstHrdwFlow.SelectedIndex);
                serialPort.SoftwareFlowControl = (SerialSoftwareFlowControl)(lstSoftFlow.SelectedIndex);
                serialPort.SetDtr(checkBoxDTR.Checked);                
                serialPort.SetRts(checkBoxRTS.Checked);                    

            }
        }

        private void serialPort_Connected(object sender, ConnectionEventArgs e)
        {
            Set_textBoxOut(textBoxOut.Text + "Connected to: " + e.Port + Environment.NewLine);
        }

        private void serialPort_Disconnected(object sender, ConnectionEventArgs e)
        {
            Set_textBoxOut(textBoxOut.Text + Environment.NewLine + "Disconnected from: " + e.Port + Environment.NewLine);
            lblCts.ForeColor = Color.Red;
            lblDsr.ForeColor = Color.Red;
            lblCd.ForeColor = Color.Red;
            lblRing.ForeColor = Color.Red;            
        }

        private void serialPort_LineStatusChanged(object sender, LineStatusEventArgs e)
        {
            if (SerialPort.GetCts(e.LineStatus))
                lblCts.ForeColor = Color.Green;
            else
                lblCts.ForeColor = Color.Red;

            if (SerialPort.GetDsr(e.LineStatus))
                lblDsr.ForeColor = Color.Green;
            else
                lblDsr.ForeColor = Color.Red;

            if (SerialPort.GetDcd(e.LineStatus))
                lblCd.ForeColor = Color.Green;
            else
                lblCd.ForeColor = Color.Red;

            if (SerialPort.GetRing(e.LineStatus))
                lblRing.ForeColor = Color.Green;
            else
                lblRing.ForeColor = Color.Red;
        }

        private void serialPort_Faulted(object sender, ConnectionEventArgs e)
        {
            Set_textBoxOut(textBoxOut.Text + Environment.NewLine + "Faulted: " + e.Port + Environment.NewLine);                        
        } 

    
    }
}
