using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;
using ZylSoft.Serial;

namespace CSharpDemo
{	
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public partial class Form1 : System.Windows.Forms.Form
    {
        private ZylSoft.Serial.SerialPort serialPort;
		                     
		private System.ComponentModel.IContainer components;

        // This delegate enables asynchronous calls for setting
        // the text property on a TextBox control.
        delegate void SetTextCallback(string text);

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			//
			// TODO: Add any constructor code after InitializeComponent call
			//

            // 
            // serialPort
            // 
            serialPort = new SerialPort();
            this.serialPort.AutoReceive = true;
            this.serialPort.AutoReconnect = false;
            this.serialPort.AutoReconnectCheckInterval = 4000;
            this.serialPort.BaudRate = ZylSoft.Serial.SerialBaudRate.Br004800;
            this.serialPort.CloseWhenLineStatusIsZero = true;
            this.serialPort.CustomBaudRate = ((uint)(0u));
            this.serialPort.CustomPortName = "port1";
            this.serialPort.DataWidth = ZylSoft.Serial.SerialDataWidth.Dw8Bits;            
            this.serialPort.DiscardNulls = false;
            this.serialPort.DsrSensitivity = false;
            this.serialPort.EnableDtrOnOpen = false;
            this.serialPort.EnableRtsOnOpen = false;
            this.serialPort.EofChar = ((uint)(0u));
            this.serialPort.ErrorChar = ((uint)(0u));
            this.serialPort.EvtChar = ((uint)(26u));
            this.serialPort.HardwareFlowControl = ZylSoft.Serial.SerialHardwareFlowControl.None;
            this.serialPort.IdleInterval = 10000;
            this.serialPort.InputBuffer = 4096;
            this.serialPort.Interval = 1000;
            this.serialPort.LogDateTime = true;
            this.serialPort.LogFile = "";
            this.serialPort.LogInput = true;
            this.serialPort.MaxLineLength = 0;
            this.serialPort.MinCheckDisconnectedInterval = 50;
            this.serialPort.NeedSynchronization = false;
            this.serialPort.NewLine = "\r\n";
            this.serialPort.OutputBuffer = 4096;
            this.serialPort.ParityBits = ZylSoft.Serial.SerialParityBits.None;
            this.serialPort.ParityCheck = false;
            this.serialPort.Port = ZylSoft.Serial.SerialCommPort.COM2;
            this.serialPort.Priority = System.Threading.ThreadPriority.Normal;
            this.serialPort.ReadIntervalTimeout = ((uint)(4294967295u));
            this.serialPort.ReadTotalTimeoutConstant = ((uint)(0u));
            this.serialPort.ReadTotalTimeoutMultiplier = ((uint)(0u));
            this.serialPort.ReplaceParityErrors = false;
            this.serialPort.SkipSetupFailed = false;
            this.serialPort.SoftwareFlowControl = ZylSoft.Serial.SerialSoftwareFlowControl.None;
            this.serialPort.StopBits = ZylSoft.Serial.SerialStopBits.Sb1Bit;
            this.serialPort.TxContinueOnXoff = false;
            this.serialPort.WriteTotalTimeoutConstant = 1000;
            this.serialPort.WriteTotalTimeoutMultiplier = 100;
            this.serialPort.XoffChar = ((uint)(19u));
            this.serialPort.XoffLim = ((ushort)(1));
            this.serialPort.XonChar = ((uint)(17u));
            this.serialPort.XonLim = ((ushort)(0));
            this.serialPort.Received += new System.EventHandler<ZylSoft.Serial.DataEventArgs>(this.serialPort_Received);
            this.serialPort.Sent += new System.EventHandler<ZylSoft.Serial.DataEventArgs>(this.serialPort_Sent);
            this.serialPort.Connected += new System.EventHandler<ZylSoft.Serial.ConnectionEventArgs>(this.serialPort_Connected);
            this.serialPort.Disconnected += new System.EventHandler<ZylSoft.Serial.ConnectionEventArgs>(this.serialPort_Disconnected);
            this.serialPort.Faulted += new System.EventHandler<ZylSoft.Serial.ConnectionEventArgs>(this.serialPort_Faulted);
            this.serialPort.LineStatusChanged += new System.EventHandler<ZylSoft.Serial.LineStatusEventArgs>(this.serialPort_LineStatusChanged);
            this.serialPort.Idle += new System.EventHandler<System.EventArgs>(this.serialPort_Idle);
            this.serialPort.Resumed += new System.EventHandler<System.EventArgs>(this.serialPort_Resume);
		}		

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

        private void Set_textBoxOut(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.textBoxOut.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(Set_textBoxOut);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.textBoxOut.AppendText(text);
                this.textBoxOut.SelectionStart = textBoxOut.Text.Length;
                this.textBoxOut.SelectionLength = 0;
                this.textBoxOut.ScrollToCaret();
            }
        }


		private void btnOpen_Click(object sender, System.EventArgs e)
		{
		    SerialPort.UnlockKey = "";
            serialPort.AutoReconnect = checkBoxReconnect.Checked;
            if (lstPorts.SelectedIndex < 0)
            {
                MessageBox.Show("Select a serial port!");
                return;
            }            
            serialPort.Port = SerialPort.StringToSerialCommPort(lstPorts.Items[lstPorts.SelectedIndex].ToString());
            if (serialPort.Port == SerialCommPort.Custom)
            {
                serialPort.CustomPortName = lstPorts.Items[lstPorts.SelectedIndex].ToString();
            }            

            serialPort.BaudRate = (SerialBaudRate)(Convert.ToInt32(lstBaudRates.Items[lstBaudRates.SelectedIndex].ToString()));
            serialPort.DataWidth = (SerialDataWidth)(lstDataWidth.SelectedIndex + 5);
            serialPort.ParityBits = (SerialParityBits)(lstParity.SelectedIndex);
            serialPort.StopBits = (SerialStopBits)(lstStopBits.SelectedIndex);
            serialPort.HardwareFlowControl = (SerialHardwareFlowControl)(lstHrdwFlow.SelectedIndex);
            serialPort.SoftwareFlowControl = (SerialSoftwareFlowControl)(lstSoftFlow.SelectedIndex);
            if (checkBoxDTR.Checked)
                serialPort.EnableDtrOnOpen = true;
            else
                serialPort.EnableDtrOnOpen = false;
            if (checkBoxRTS.Checked)
                serialPort.EnableRtsOnOpen = true;
            else
                serialPort.EnableRtsOnOpen = false;
            
            if (!serialPort.Open())
                MessageBox.Show("Cannot connect to " + serialPort.Port.ToString());
		}

		private void btnClose_Click(object sender, System.EventArgs e)
		{
			serialPort.Close();
		}

		private void btnSend_Click(object sender, System.EventArgs e)
		{
			serialPort.SendAsciiString(txtText.Text + Environment.NewLine);
		}		

		private void serialPort_Sent(object sender, ZylSoft.Serial.DataEventArgs e)
		{
			Set_textBoxOut("Sent data: " + SerialPort.AsciiByteArrayToString(e.Buffer) + Environment.NewLine);			
		}

		private void serialPort_Received(object sender, ZylSoft.Serial.DataEventArgs e)
		{
			if (rbAscii.Checked)
			{
			    if (!checkBoxShowHiddenChars.Checked)
			        Set_textBoxOut(SerialPort.AsciiByteArrayToString(e.Buffer));
                else
                    Set_textBoxOut(SerialPort.ReplaceControlCharacters(SerialPort.AsciiByteArrayToString(e.Buffer)));
			}
			else if (rbDecimal.Checked)
			{
				Set_textBoxOut(SerialPort.ByteArrayToDecimalString(e.Buffer));
			}
			else
			{
				Set_textBoxOut(SerialPort.ByteArrayToHexaString(e.Buffer));
			}			
		}

        private void Form1_Load(object sender, EventArgs e)
        {
            lstBaudRates.SelectedIndex = 9;
            lstDataWidth.SelectedIndex = 3;
            lstParity.SelectedIndex = 0;
            lstStopBits.SelectedIndex = 0;
            lstHrdwFlow.SelectedIndex = 0;
            lstSoftFlow.SelectedIndex = 0;

            lstPorts.Items.Clear();
            String[] ports = ZylSoft.Serial.SerialPort.GetExistingCommPortNames();
            foreach (String i in ports)
            {
                lstPorts.Items.Add(i);  
            }
        }

        private void btnApply_Click(object sender, EventArgs e)
        {
            if (serialPort.ConnectedTo != SerialCommPort.None)
            {
                serialPort.BaudRate = (SerialBaudRate)(Convert.ToInt32(lstBaudRates.Items[lstBaudRates.SelectedIndex].ToString()));
                serialPort.DataWidth = (SerialDataWidth)(lstDataWidth.SelectedIndex + 5);
                serialPort.ParityBits = (SerialParityBits)(lstParity.SelectedIndex);
                serialPort.StopBits = (SerialStopBits)(lstStopBits.SelectedIndex);
                serialPort.HardwareFlowControl = (SerialHardwareFlowControl)(lstHrdwFlow.SelectedIndex);
                serialPort.SoftwareFlowControl = (SerialSoftwareFlowControl)(lstSoftFlow.SelectedIndex);
                serialPort.SetDtr(checkBoxDTR.Checked);                
                serialPort.SetRts(checkBoxRTS.Checked);                    

            }
        }

        private void serialPort_Connected(object sender, ConnectionEventArgs e)
        {
            Set_textBoxOut("Connected to: " + e.Port + Environment.NewLine);
        }

        private void serialPort_Disconnected(object sender, ConnectionEventArgs e)
        {
            Set_textBoxOut("Disconnected from: " + e.Port + Environment.NewLine);
            lblCts.ForeColor = Color.Red;
            lblDsr.ForeColor = Color.Red;
            lblCd.ForeColor = Color.Red;
            lblRing.ForeColor = Color.Red;            
        }

        private void serialPort_LineStatusChanged(object sender, LineStatusEventArgs e)
        {
            if (SerialPort.GetCts(e.LineStatus))
                lblCts.ForeColor = Color.Green;
            else
                lblCts.ForeColor = Color.Red;

            if (SerialPort.GetDsr(e.LineStatus))
                lblDsr.ForeColor = Color.Green;
            else
                lblDsr.ForeColor = Color.Red;

            if (SerialPort.GetDcd(e.LineStatus))
                lblCd.ForeColor = Color.Green;
            else
                lblCd.ForeColor = Color.Red;

            if (SerialPort.GetRing(e.LineStatus))
                lblRing.ForeColor = Color.Green;
            else
                lblRing.ForeColor = Color.Red;
        }

        private void serialPort_Faulted(object sender, ConnectionEventArgs e)
        {
            Set_textBoxOut("Faulted: " + e.Port + Environment.NewLine);                        
        }

        private void serialPort_Idle(object sender, EventArgs e)
        {
            Set_textBoxOut("Idle" + Environment.NewLine);
        }

        private void serialPort_Resume(object sender, EventArgs e)
        {
            Set_textBoxOut("Resume" + Environment.NewLine);
        }       

        private void buttonDetect_Click(object sender, EventArgs e)
        {
            SerialCommPort port;
            SerialBaudRate baudRate;
            if (serialPort.DetectDevice("AT\r", "OK", out port, out baudRate))
            //if (serialPort.DetectDevice("", "$GP", out port, out baudRate))
            {
                MessageBox.Show("Port: " + serialPort.SerialCommPortToString(port) + ", Baud Rate: " + baudRate.ToString());
                lstPorts.SelectedIndex = lstPorts.Items.IndexOf(serialPort.SerialCommPortToString(port));
                lstBaudRates.SelectedIndex = lstBaudRates.Items.IndexOf(((int)baudRate).ToString());
            }
            else
                MessageBox.Show("No port detected.");
            //serialPort.SendByteArray(new byte[5] {(byte) 66, (byte) 67, (byte) 68, (byte) 69, (byte) 70}, 2, 2);
            //serialPort.SendCharArray(new char[3] {'A', 'B', 'C'});

        }

        private void checkBoxReconnect_CheckedChanged(object sender, EventArgs e)
        {
            //timerReconnect.Enabled = checkBoxReconnect.Checked;
            serialPort.AutoReconnect = checkBoxReconnect.Checked;
        }

        private void timerReconnect_Tick(object sender, EventArgs e)
        {
            if (serialPort.IsFaulted) // || !serialPort.IsConnected())
            {
                serialPort.Open();
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (serialPort != null)
            {
                if (serialPort.IsConnected())
                    serialPort.Close();
                serialPort.Dispose();
            }
        } 

    
    }
}
